# pylint: disable=line-too-long
"""Constants for the Notification CC."""

# ----------------------------------------------------------------------------------- #
# **BEGINNING OF AUTOGENERATED CONTENT** (TO ADD ADDITIONAL MANUAL CONTENT, LOOK FOR  #
# THE "END OF AUTOGENERATED CONTENT" COMMENT BLOCK AND ADD YOUR CODE BELOW IT)        #
# ----------------------------------------------------------------------------------- #

from __future__ import annotations

from enum import IntEnum

CC_SPECIFIC_NOTIFICATION_TYPE = "notificationType"


class NotificationType(IntEnum):
    """Enum for known notification types."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    ACCESS_CONTROL = 6
    APPLIANCE = 12
    CLOCK = 11
    CO_ALARM = 2
    CO2_ALARM = 3
    EMERGENCY_ALARM = 10
    GAS_ALARM = 18
    HEAT_ALARM = 4
    HOME_HEALTH = 13
    HOME_MONITORING = 22
    HOME_SECURITY = 7
    IRRIGATION = 17
    LIGHT_SENSOR = 20
    PEST_CONTROL = 19
    POWER_MANAGEMENT = 8
    SIREN = 14
    SMOKE_ALARM = 1
    SYSTEM = 9
    WATER_ALARM = 5
    WATER_QUALITY_MONITORING = 21
    WATER_VALVE = 15
    WEATHER_ALARM = 16

    @classmethod
    def _missing_(cls: type, value: object) -> NotificationType:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return NotificationType.UNKNOWN


class NotificationEvent(IntEnum):
    """Common base class for Notification CC states enums."""


class NotificationEventValue(IntEnum):
    """Common base class for Notification CC state value enums."""


class AccessControlNotificationEvent(NotificationEvent):
    """Enum for known access control notification event."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    ALL_USER_CODES_DELETED = 12
    ALL_USERS_DELETED = 37
    AUTO_LOCK_LOCKED_OPERATION = 9
    AUTO_LOCK_NOT_FULLY_LOCKED_OPERATION = 10
    BARRIER_BATTERY_STATUS_BARRIER_SENSOR_LOW_BATTERY_WARNING = 74
    BARRIER_CONTROL_STATUS_BARRIER_ASSOCIATED_WITH_NON_Z_WAVE_REMOTE_CONTROL = 76
    BARRIER_FAILED_TO_PERFORM_REQUESTED_OPERATION_DEVICE_MALFUNCTION = 70
    BARRIER_MOTOR_HAS_EXCEEDED_MANUFACTURER_S_OPERATIONAL_TIME_LIMIT = 66
    BARRIER_OPERATION_OPEN_CLOSE_FORCE_HAS_BEEN_EXCEEDED = 65
    BARRIER_OPERATION_HAS_EXCEEDED_PHYSICAL_MECHANICAL_LIMITS = 67
    BARRIER_PERFORMING_INITIALIZATION_PROCESS = 64
    BARRIER_SAFETY_BEAM_OBSTACLE = 72
    BARRIER_SENSOR_STATUS_BARRIER_SENSOR_NOT_DETECTED_SUPERVISORY_ERROR = 73
    BARRIER_SHORT_CIRCUIT_STATUS_BARRIER_DETECTED_SHORT_IN_WALL_STATION_WIRES = 75
    BARRIER_UL_DISABLING_STATUS_BARRIER_UNATTENDED_OPERATION_HAS_BEEN_DISABLED_PER_UL_REQUIREMENTS = (
        69
    )
    BARRIER_UNABLE_TO_PERFORM_REQUESTED_OPERATION_DUE_TO_UL_REQUIREMENTS = 68
    BARRIER_VACATION_MODE = 71
    CREDENTIAL_ADDED = 43
    CREDENTIAL_DELETED = 45
    CREDENTIAL_LOCK_OPEN_OPERATION = 35
    CREDENTIAL_MODIFIED = 44
    CREDENTIAL_UNCHANGED = 46
    CREDENTIAL_UNLOCK_CLOSE_OPERATION = 36
    DOOR_HANDLE_STATE_WINDOW_DOOR_HANDLE_IS_CLOSED = 25
    DOOR_HANDLE_STATE_WINDOW_DOOR_HANDLE_IS_OPEN = 24
    DOOR_STATE_WINDOW_DOOR_IS_CLOSED = 23
    DOOR_STATE_WINDOW_DOOR_IS_OPEN = 22
    INVALID_CREDENTIAL_USED = 50
    KEYPAD_LOCK_OPERATION = 5
    KEYPAD_STATE_KEYPAD_BUSY = 17
    KEYPAD_STATE_KEYPAD_TEMPORARY_DISABLED = 16
    KEYPAD_UNLOCK_OPERATION = 6
    LOCK_OPERATION_WITH_USER_CODE = 33
    LOCK_STATE_LOCK_JAMMED = 11
    LOCKED_BY_RF_WITH_INVALID_USER_CODE = 21
    MANUAL_LOCK_OPERATION = 1
    MANUAL_NOT_FULLY_LOCKED_OPERATION = 7
    MANUAL_UNLOCK_OPERATION = 2
    MANUALLY_ENTER_USER_ACCESS_CODE_EXCEEDS_CODE_LIMIT = 19
    MESSAGING_USER_CODE_ENTERED_VIA_KEYPAD = 32
    MULTIPLE_CREDENTIALS_DELETED = 38
    NEW_PROGRAM_CODE_ENTERED_UNIQUE_CODE_FOR_LOCK_CONFIGURATION = 18
    NEW_USER_CODE_ADDED = 14
    NEW_USER_CODE_NOT_ADDED_DUE_TO_DUPLICATE_CODE = 15
    NON_ACCESS_CREDENTIAL_ENTERED_VIA_LOCAL_INTERFACE = 51
    RF_LOCK_OPERATION = 3
    RF_NOT_FULLY_LOCKED_OPERATION = 8
    RF_UNLOCK_OPERATION = 4
    SINGLE_USER_CODE_DELETED = 13
    UNLOCK_BY_RF_WITH_INVALID_USER_CODE = 20
    UNLOCK_OPERATION_WITH_USER_CODE = 34
    USER_ADDED = 39
    USER_DELETED = 41
    USER_MODIFIED = 40
    USER_UNCHANGED = 42
    VALID_CREDENTIAL_ACCESS_DENIED_NOT_ENOUGH_CREDENTIALS_ENTERED = 49
    VALID_CREDENTIAL_ACCESS_DENIED_SCHEDULE_INACTIVE = 48
    VALID_CREDENTIAL_ACCESS_DENIED_USER_ACTIVE_STATE_SET_TO_OCCUPIED_DISABLED = 47

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> AccessControlNotificationEvent:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return AccessControlNotificationEvent.UNKNOWN


class BarrierPerformingInitializationProcessNotificationEventValue(
    NotificationEventValue
):
    """Enum for known barrier performing initialization process notification event value."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    PERFORMING_PROCESS = 255
    PROCESS_COMPLETED = 0

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> BarrierPerformingInitializationProcessNotificationEventValue:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return BarrierPerformingInitializationProcessNotificationEventValue.UNKNOWN


class BarrierSafetyBeamObstacleNotificationEventValue(NotificationEventValue):
    """Enum for known barrier safety beam obstacle notification event value."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    NO_OBSTRUCTION = 0
    OBSTRUCTION = 255

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> BarrierSafetyBeamObstacleNotificationEventValue:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return BarrierSafetyBeamObstacleNotificationEventValue.UNKNOWN


class BarrierVacationModeNotificationEventValue(NotificationEventValue):
    """Enum for known barrier vacation mode notification event value."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    MODE_DISABLED = 0
    MODE_ENABLED = 255

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> BarrierVacationModeNotificationEventValue:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return BarrierVacationModeNotificationEventValue.UNKNOWN


class DoorStateWindowDoorIsOpenNotificationEventValue(NotificationEventValue):
    """Enum for known door state window/door is open notification event value."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    WINDOW_DOOR_IS_OPEN_IN_REGULAR_POSITION = 0
    WINDOW_DOOR_IS_OPEN_IN_TILT_POSITION = 1

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> DoorStateWindowDoorIsOpenNotificationEventValue:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return DoorStateWindowDoorIsOpenNotificationEventValue.UNKNOWN


class ApplianceNotificationEvent(NotificationEvent):
    """Enum for known appliance notification event."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    APPLIANCE_STATUS_BOILING = 8
    APPLIANCE_STATUS_DRAINING = 14
    APPLIANCE_STATUS_DRYING = 18
    APPLIANCE_STATUS_RINSING = 12
    APPLIANCE_STATUS_SPINNING = 16
    APPLIANCE_STATUS_SUPPLYING_WATER = 6
    APPLIANCE_STATUS_WASHING = 10
    BOILING_FAILURE = 9
    COMPRESSOR_FAILURE = 21
    DRAINING_FAILURE = 15
    DRYING_FAILURE = 19
    FAN_FAILURE = 20
    MAINTENANCE_STATUS_REPLACE_MAIN_FILTER = 4
    PROGRAM_STATUS_PROGRAM_COMPLETED = 3
    PROGRAM_STATUS_PROGRAM_IN_PROGRESS = 2
    PROGRAM_STATUS_PROGRAM_STARTED = 1
    RINSING_FAILURE = 13
    SPINNING_FAILURE = 17
    TARGET_TEMPERATURE_FAILURE_STATUS_FAILURE_TO_SET_TARGET_TEMPERATURE = 5
    WASHING_FAILURE = 11
    WATER_SUPPLY_FAILURE = 7

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> ApplianceNotificationEvent:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return ApplianceNotificationEvent.UNKNOWN


class ClockNotificationEvent(NotificationEvent):
    """Enum for known clock notification event."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    TIME_REMAINING = 3
    TIMER_ENDED = 2
    WAKE_UP_ALERT = 1

    @classmethod
    def _missing_(cls: type, value: object) -> ClockNotificationEvent:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return ClockNotificationEvent.UNKNOWN


class CoAlarmNotificationEvent(NotificationEvent):
    """Enum for known co alarm notification event."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    ALARM_STATUS_ALARM_SILENCED = 6
    MAINTENANCE_STATUS_REPLACEMENT_REQUIRED = 4
    MAINTENANCE_STATUS_REPLACEMENT_REQUIRED_END_OF_LIFE = 5
    PERIODIC_INSPECTION_STATUS_MAINTENANCE_REQUIRED_PLANNED_PERIODIC_INSPECTION = 7
    SENSOR_STATUS_CARBON_MONOXIDE_DETECTED = 2
    SENSOR_STATUS_CARBON_MONOXIDE_DETECTED_LOCATION_PROVIDED = 1
    TEST_STATUS_CARBON_MONOXIDE_TEST = 3

    @classmethod
    def _missing_(cls: type, value: object) -> CoAlarmNotificationEvent:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return CoAlarmNotificationEvent.UNKNOWN


class TestStatusCarbonMonoxideTestNotificationEventValue(NotificationEventValue):
    """Enum for known test status carbon monoxide test notification event value."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    TEST_FAILED = 2
    TEST_OK = 1

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> TestStatusCarbonMonoxideTestNotificationEventValue:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return TestStatusCarbonMonoxideTestNotificationEventValue.UNKNOWN


class Co2AlarmNotificationEvent(NotificationEvent):
    """Enum for known co2 alarm notification event."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    ALARM_STATUS_ALARM_SILENCED = 6
    MAINTENANCE_STATUS_REPLACEMENT_REQUIRED = 4
    MAINTENANCE_STATUS_REPLACEMENT_REQUIRED_END_OF_LIFE = 5
    PERIODIC_INSPECTION_STATUS_MAINTENANCE_REQUIRED_PLANNED_PERIODIC_INSPECTION = 7
    SENSOR_STATUS_CARBON_DIOXIDE_DETECTED = 2
    SENSOR_STATUS_CARBON_DIOXIDE_DETECTED_LOCATION_PROVIDED = 1
    TEST_STATUS_CARBON_DIOXIDE_TEST = 3

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> Co2AlarmNotificationEvent:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return Co2AlarmNotificationEvent.UNKNOWN


class TestStatusCarbonDioxideTestNotificationEventValue(NotificationEventValue):
    """Enum for known test status carbon dioxide test notification event value."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    TEST_FAILED = 2
    TEST_OK = 1

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> TestStatusCarbonDioxideTestNotificationEventValue:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return TestStatusCarbonDioxideTestNotificationEventValue.UNKNOWN


class EmergencyAlarmNotificationEvent(NotificationEvent):
    """Enum for known emergency alarm notification event."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    CONTACT_FIRE_SERVICE = 2
    CONTACT_MEDICAL_SERVICE = 3
    CONTACT_POLICE = 1

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> EmergencyAlarmNotificationEvent:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return EmergencyAlarmNotificationEvent.UNKNOWN


class GasAlarmNotificationEvent(NotificationEvent):
    """Enum for known gas alarm notification event."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    ALARM_STATUS_GAS_ALARM_TEST = 5
    COMBUSTIBLE_GAS_STATUS_COMBUSTIBLE_GAS_DETECTED = 2
    COMBUSTIBLE_GAS_STATUS_COMBUSTIBLE_GAS_DETECTED_LOCATION_PROVIDED = 1
    MAINTENANCE_STATUS_REPLACEMENT_REQUIRED = 6
    TOXIC_GAS_STATUS_TOXIC_GAS_DETECTED = 4
    TOXIC_GAS_STATUS_TOXIC_GAS_DETECTED_LOCATION_PROVIDED = 3

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> GasAlarmNotificationEvent:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return GasAlarmNotificationEvent.UNKNOWN


class HeatAlarmNotificationEvent(NotificationEvent):
    """Enum for known heat alarm notification event."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    ALARM_STATUS_ALARM_SILENCED = 9
    ALARM_STATUS_HEAT_ALARM_TEST = 7
    DUST_IN_DEVICE_STATUS_MAINTENANCE_REQUIRED_DUST_IN_DEVICE = 10
    HEAT_SENSOR_STATUS_OVERHEAT_DETECTED = 2
    HEAT_SENSOR_STATUS_OVERHEAT_DETECTED_LOCATION_PROVIDED = 1
    HEAT_SENSOR_STATUS_UNDERHEAT_DETECTED = 6
    HEAT_SENSOR_STATUS_UNDERHEAT_DETECTED_LOCATION_PROVIDED = 5
    MAINTENANCE_STATUS_REPLACEMENT_REQUIRED_END_OF_LIFE = 8
    PERIODIC_INSPECTION_STATUS_MAINTENANCE_REQUIRED_PLANNED_PERIODIC_INSPECTION = 11
    RAPID_TEMPERATURE_FALL = 13
    RAPID_TEMPERATURE_FALL_LOCATION_PROVIDED = 12
    RAPID_TEMPERATURE_RISE = 4
    RAPID_TEMPERATURE_RISE_LOCATION_PROVIDED = 3

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> HeatAlarmNotificationEvent:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return HeatAlarmNotificationEvent.UNKNOWN


class HomeHealthNotificationEvent(NotificationEvent):
    """Enum for known home health notification event."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    FALL_DETECTED = 12
    POSITION_STATUS_LEAVING_BED = 1
    POSITION_STATUS_LYING_ON_BED = 3
    POSITION_STATUS_SITTING_ON_BED = 2
    POSITION_STATUS_SITTING_ON_BED_EDGE = 5
    POSTURE_CHANGED = 4
    SLEEP_APNEA_STATUS_SLEEP_APNEA_DETECTED = 7
    SLEEP_STAGE_STATUS_SLEEP_STAGE_0_DETECTED_DREAMING_REM = 8
    SLEEP_STAGE_STATUS_SLEEP_STAGE_1_DETECTED_LIGHT_SLEEP_NON_REM_1 = 9
    SLEEP_STAGE_STATUS_SLEEP_STAGE_2_DETECTED_MEDIUM_SLEEP_NON_REM_2 = 10
    SLEEP_STAGE_STATUS_SLEEP_STAGE_3_DETECTED_DEEP_SLEEP_NON_REM_3 = 11
    VOC_LEVEL_STATUS_VOLATILE_ORGANIC_COMPOUND_LEVEL = 6

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> HomeHealthNotificationEvent:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return HomeHealthNotificationEvent.UNKNOWN


class SleepApneaStatusSleepApneaDetectedNotificationEventValue(NotificationEventValue):
    """Enum for known sleep apnea status sleep apnea detected notification event value."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    LOW_BREATH = 1
    NO_BREATH_AT_ALL = 2

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> SleepApneaStatusSleepApneaDetectedNotificationEventValue:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return SleepApneaStatusSleepApneaDetectedNotificationEventValue.UNKNOWN


class VocLevelStatusVolatileOrganicCompoundLevelNotificationEventValue(
    NotificationEventValue
):
    """Enum for known voc level status volatile organic compound level notification event value."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    CLEAN = 1
    HIGHLY_POLLUTED = 4
    MODERATELY_POLLUTED = 3
    SLIGHTLY_POLLUTED = 2

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> (
        VocLevelStatusVolatileOrganicCompoundLevelNotificationEventValue
    ):  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return VocLevelStatusVolatileOrganicCompoundLevelNotificationEventValue.UNKNOWN


class HomeMonitoringNotificationEvent(NotificationEvent):
    """Enum for known home monitoring notification event."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    HOME_OCCUPANCY_STATUS_HOME_OCCUPIED = 2
    HOME_OCCUPANCY_STATUS_HOME_OCCUPIED_LOCATION_PROVIDED = 1

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> HomeMonitoringNotificationEvent:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return HomeMonitoringNotificationEvent.UNKNOWN


class HomeSecurityNotificationEvent(NotificationEvent):
    """Enum for known home security notification event."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    COVER_STATUS_TAMPERING_PRODUCT_COVER_REMOVED = 3
    GLASS_BREAKAGE = 6
    GLASS_BREAKAGE_LOCATION_PROVIDED = 5
    IMPACT_DETECTED = 10
    MAGNETIC_INTERFERENCE_STATUS_MAGNETIC_FIELD_INTERFERENCE_DETECTED = 11
    MOTION_SENSOR_STATUS_MOTION_DETECTION = 8
    MOTION_SENSOR_STATUS_MOTION_DETECTION_LOCATION_PROVIDED = 7
    RF_JAMMING_DETECTED = 12
    SENSOR_STATUS_INTRUSION = 2
    SENSOR_STATUS_INTRUSION_LOCATION_PROVIDED = 1
    TAMPERING_INVALID_CODE = 4
    TAMPERING_PRODUCT_MOVED = 9

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> HomeSecurityNotificationEvent:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return HomeSecurityNotificationEvent.UNKNOWN


class IrrigationNotificationEvent(NotificationEvent):
    """Enum for known irrigation notification event."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    DEVICE_CONFIGURATION_STATUS_DEVICE_IS_NOT_CONFIGURED = 5
    SCHEDULE_ID_STATUS_SCHEDULE_FINISHED = 2
    SCHEDULE_ID_STATUS_SCHEDULE_STARTED = 1
    VALVE_ID_RUN_STATUS_VALVE_TABLE_RUN_FINISHED = 4
    VALVE_ID_RUN_STATUS_VALVE_TABLE_RUN_STARTED = 3

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> IrrigationNotificationEvent:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return IrrigationNotificationEvent.UNKNOWN


class LightSensorNotificationEvent(NotificationEvent):
    """Enum for known light sensor notification event."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    LIGHT_COLOR_TRANSITION_DETECTED = 2
    LIGHT_DETECTION_STATUS_LIGHT_DETECTED = 1

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> LightSensorNotificationEvent:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return LightSensorNotificationEvent.UNKNOWN


class PestControlNotificationEvent(NotificationEvent):
    """Enum for known pest control notification event."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    PEST_DETECTED = 6
    PEST_DETECTED_LOCATION_PROVIDED = 5
    PEST_EXTERMINATED = 8
    PEST_EXTERMINATED_LOCATION_PROVIDED = 7
    TRAP_STATUS_TRAP_ARMED = 2
    TRAP_STATUS_TRAP_ARMED_LOCATION_PROVIDED = 1
    TRAP_STATUS_TRAP_RE_ARM_REQUIRED = 4
    TRAP_STATUS_TRAP_RE_ARM_REQUIRED_LOCATION_PROVIDED = 3

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> PestControlNotificationEvent:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return PestControlNotificationEvent.UNKNOWN


class PowerManagementNotificationEvent(NotificationEvent):
    """Enum for known power management notification event."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    BACKUP_BATTERY_LEVEL_STATUS_BACK_UP_BATTERY_DISCONNECTED = 18
    BACKUP_BATTERY_LEVEL_STATUS_BACK_UP_BATTERY_IS_LOW = 16
    BATTERY_LEVEL_STATUS_BATTERY_IS_FULLY_CHARGED = 13
    BATTERY_LEVEL_STATUS_CHARGE_BATTERY_NOW = 15
    BATTERY_LEVEL_STATUS_CHARGE_BATTERY_SOON = 14
    BATTERY_LOAD_STATUS_BATTERY_IS_CHARGING = 12
    BATTERY_MAINTENANCE_STATUS_BATTERY_FLUID_IS_LOW = 17
    BATTERY_MAINTENANCE_STATUS_REPLACE_BATTERY_NOW = 11
    BATTERY_MAINTENANCE_STATUS_REPLACE_BATTERY_SOON = 10
    LOAD_ERROR = 9
    MAINS_STATUS_AC_MAINS_DISCONNECTED = 2
    MAINS_STATUS_AC_MAINS_RE_CONNECTED = 3
    OVER_CURRENT_STATUS_OVER_CURRENT_DETECTED = 6
    OVER_LOAD_STATUS_OVER_LOAD_DETECTED = 8
    OVER_VOLTAGE_STATUS_OVER_VOLTAGE_DETECTED = 7
    POWER_STATUS_POWER_HAS_BEEN_APPLIED = 1
    SURGE_DETECTED = 4
    VOLTAGE_DROP_DRIFT = 5

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> PowerManagementNotificationEvent:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return PowerManagementNotificationEvent.UNKNOWN


class SirenNotificationEvent(NotificationEvent):
    """Enum for known siren notification event."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    SIREN_STATUS_SIREN_ACTIVE = 1

    @classmethod
    def _missing_(cls: type, value: object) -> SirenNotificationEvent:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return SirenNotificationEvent.UNKNOWN


class SmokeAlarmNotificationEvent(NotificationEvent):
    """Enum for known smoke alarm notification event."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    ALARM_STATUS_ALARM_SILENCED = 6
    ALARM_STATUS_SMOKE_ALARM_TEST = 3
    DUST_IN_DEVICE_STATUS_MAINTENANCE_REQUIRED_DUST_IN_DEVICE = 8
    MAINTENANCE_STATUS_REPLACEMENT_REQUIRED = 4
    MAINTENANCE_STATUS_REPLACEMENT_REQUIRED_END_OF_LIFE = 5
    PERIODIC_INSPECTION_STATUS_MAINTENANCE_REQUIRED_PLANNED_PERIODIC_INSPECTION = 7
    SENSOR_STATUS_SMOKE_DETECTED = 2
    SENSOR_STATUS_SMOKE_DETECTED_LOCATION_PROVIDED = 1

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> SmokeAlarmNotificationEvent:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return SmokeAlarmNotificationEvent.UNKNOWN


class SystemNotificationEvent(NotificationEvent):
    """Enum for known system notification event."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    COVER_STATUS_TAMPERING_PRODUCT_COVER_REMOVED = 6
    EMERGENCY_SHUTOFF = 7
    HARDWARE_STATUS_SYSTEM_HARDWARE_FAILURE = 1
    HARDWARE_STATUS_SYSTEM_HARDWARE_FAILURE_WITH_FAILURE_CODE = 3
    HEARTBEAT = 5
    SOFTWARE_STATUS_SYSTEM_SOFTWARE_FAILURE = 2
    SOFTWARE_STATUS_SYSTEM_SOFTWARE_FAILURE_WITH_FAILURE_CODE = 4

    @classmethod
    def _missing_(cls: type, value: object) -> SystemNotificationEvent:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return SystemNotificationEvent.UNKNOWN


class WaterAlarmNotificationEvent(NotificationEvent):
    """Enum for known water alarm notification event."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    MAINTENANCE_STATUS_REPLACE_WATER_FILTER = 5
    PUMP_STATUS_SUMP_PUMP_ACTIVE = 10
    PUMP_STATUS_SUMP_PUMP_FAILURE = 11
    SENSOR_STATUS_WATER_LEAK_DETECTED = 2
    SENSOR_STATUS_WATER_LEAK_DETECTED_LOCATION_PROVIDED = 1
    WATER_FLOW_ALARM = 6
    WATER_LEVEL_ALARM = 9
    WATER_LEVEL_DROPPED = 4
    WATER_LEVEL_DROPPED_LOCATION_PROVIDED = 3
    WATER_PRESSURE_ALARM = 7
    WATER_TEMPERATURE_ALARM = 8

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> WaterAlarmNotificationEvent:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return WaterAlarmNotificationEvent.UNKNOWN


class WaterFlowAlarmNotificationEventValue(NotificationEventValue):
    """Enum for known water flow alarm notification event value."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    ABOVE_HIGH_THRESHOLD = 3
    BELOW_LOW_THRESHOLD = 2
    MAX = 4
    NO_DATA = 1

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> WaterFlowAlarmNotificationEventValue:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return WaterFlowAlarmNotificationEventValue.UNKNOWN


class WaterLevelAlarmNotificationEventValue(NotificationEventValue):
    """Enum for known water level alarm notification event value."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    ABOVE_HIGH_THRESHOLD = 3
    BELOW_LOW_THRESHOLD = 2
    NO_DATA = 1

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> WaterLevelAlarmNotificationEventValue:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return WaterLevelAlarmNotificationEventValue.UNKNOWN


class WaterPressureAlarmNotificationEventValue(NotificationEventValue):
    """Enum for known water pressure alarm notification event value."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    ABOVE_HIGH_THRESHOLD = 3
    BELOW_LOW_THRESHOLD = 2
    MAX = 4
    NO_DATA = 1

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> WaterPressureAlarmNotificationEventValue:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return WaterPressureAlarmNotificationEventValue.UNKNOWN


class WaterTemperatureAlarmNotificationEventValue(NotificationEventValue):
    """Enum for known water temperature alarm notification event value."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    ABOVE_HIGH_THRESHOLD = 3
    BELOW_LOW_THRESHOLD = 2
    NO_DATA = 1

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> WaterTemperatureAlarmNotificationEventValue:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return WaterTemperatureAlarmNotificationEventValue.UNKNOWN


class WaterQualityMonitoringNotificationEvent(NotificationEvent):
    """Enum for known water quality monitoring notification event."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    ACIDITY_PH_SENSOR_STATUS_ACIDITY_PH_EMPTY = 5
    ACIDITY_PH_STATUS_ACIDITY_PH_ALARM = 2
    CHLORINE_ALARM = 1
    CHLORINE_SENSOR_STATUS_CHLORINE_EMPTY = 4
    COLLECTIVE_DISORDER = 17
    DISINFECTION_SYSTEM_STATUS_DISINFECTION_SYSTEM_ERROR_DETECTED = 8
    DRY_PROTECTION_STATUS_DRY_PROTECTION_OPERATION_ACTIVE = 13
    FILTER_CLEANING_STATUS_FILTER_CLEANING_ONGOING = 9
    FILTER_PUMP_STATUS_FILTER_PUMP_OPERATION_ONGOING = 11
    FRESHWATER_FLOW_STATUS_FRESHWATER_OPERATION_ONGOING = 12
    HEATING_STATUS_HEATING_OPERATION_ONGOING = 10
    WATER_OXIDATION_ALARM = 3
    WATER_TANK_STATUS_WATER_TANK_IS_EMPTY = 14
    WATER_TANK_STATUS_WATER_TANK_IS_FULL = 16
    WATER_TANK_STATUS_WATER_TANK_LEVEL_IS_UNKNOWN = 15
    WATERFLOW_CLEAR_WATER_SENSOR_WATERFLOW_CLEAR_WATER_SHORTAGE_DETECTED = 7
    WATERFLOW_MEASURING_STATION_SENSOR_WATERFLOW_MEASURING_STATION_SHORTAGE_DETECTED = 6

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> WaterQualityMonitoringNotificationEvent:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return WaterQualityMonitoringNotificationEvent.UNKNOWN


class AcidityStatusAcidityAlarmNotificationEventValue(NotificationEventValue):
    """Enum for known acidity (ph) status acidity (ph) alarm notification event value."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    ABOVE_HIGH_THRESHOLD = 2
    BELOW_LOW_THRESHOLD = 1
    DECREASING_PH = 3
    INCREASING_PH = 4

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> AcidityStatusAcidityAlarmNotificationEventValue:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return AcidityStatusAcidityAlarmNotificationEventValue.UNKNOWN


class ChlorineAlarmNotificationEventValue(NotificationEventValue):
    """Enum for known chlorine alarm notification event value."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    ABOVE_HIGH_THRESHOLD = 2
    BELOW_LOW_THRESHOLD = 1

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> ChlorineAlarmNotificationEventValue:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return ChlorineAlarmNotificationEventValue.UNKNOWN


class WaterOxidationAlarmNotificationEventValue(NotificationEventValue):
    """Enum for known water oxidation alarm notification event value."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    ABOVE_HIGH_THRESHOLD = 2
    BELOW_LOW_THRESHOLD = 1

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> WaterOxidationAlarmNotificationEventValue:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return WaterOxidationAlarmNotificationEventValue.UNKNOWN


class WaterValveNotificationEvent(NotificationEvent):
    """Enum for known water valve notification event."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    MASTER_VALVE_CURRENT_ALARM = 6
    MASTER_VALVE_OPERATION = 2
    MASTER_VALVE_SHORT_CIRCUIT = 4
    VALVE_CURRENT_ALARM = 5
    VALVE_JAMMED = 7
    VALVE_OPERATION = 1
    VALVE_SHORT_CIRCUIT = 3

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> WaterValveNotificationEvent:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return WaterValveNotificationEvent.UNKNOWN


class MasterValveCurrentAlarmNotificationEventValue(NotificationEventValue):
    """Enum for known master valve current alarm notification event value."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    ABOVE_HIGH_THRESHOLD = 3
    BELOW_LOW_THRESHOLD = 2
    MAX = 4
    NO_DATA = 1

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> MasterValveCurrentAlarmNotificationEventValue:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return MasterValveCurrentAlarmNotificationEventValue.UNKNOWN


class MasterValveOperationNotificationEventValue(NotificationEventValue):
    """Enum for known master valve operation notification event value."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    OFF_CLOSED = 0
    ON_OPEN = 1

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> MasterValveOperationNotificationEventValue:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return MasterValveOperationNotificationEventValue.UNKNOWN


class ValveCurrentAlarmNotificationEventValue(NotificationEventValue):
    """Enum for known valve current alarm notification event value."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    ABOVE_HIGH_THRESHOLD = 3
    BELOW_LOW_THRESHOLD = 2
    MAX = 4
    NO_DATA = 1

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> ValveCurrentAlarmNotificationEventValue:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return ValveCurrentAlarmNotificationEventValue.UNKNOWN


class ValveOperationNotificationEventValue(NotificationEventValue):
    """Enum for known valve operation notification event value."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    OFF_CLOSED = 0
    ON_OPEN = 1

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> ValveOperationNotificationEventValue:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return ValveOperationNotificationEventValue.UNKNOWN


class WeatherAlarmNotificationEvent(NotificationEvent):
    """Enum for known weather alarm notification event."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/core/src/registries/Notifications.ts
    UNKNOWN = -1
    FREEZE_ALARM = 3
    MOISTURE_ALARM = 2
    RAIN_ALARM = 1

    @classmethod
    def _missing_(
        cls: type, value: object
    ) -> WeatherAlarmNotificationEvent:  # noqa: ARG003
        """Set default enum member if an unknown value is provided."""
        return WeatherAlarmNotificationEvent.UNKNOWN


NOTIFICATION_TYPE_TO_EVENT_MAP: dict[NotificationType, type[NotificationEvent]] = {
    NotificationType.ACCESS_CONTROL: AccessControlNotificationEvent,
    NotificationType.APPLIANCE: ApplianceNotificationEvent,
    NotificationType.CLOCK: ClockNotificationEvent,
    NotificationType.CO_ALARM: CoAlarmNotificationEvent,
    NotificationType.CO2_ALARM: Co2AlarmNotificationEvent,
    NotificationType.EMERGENCY_ALARM: EmergencyAlarmNotificationEvent,
    NotificationType.GAS_ALARM: GasAlarmNotificationEvent,
    NotificationType.HEAT_ALARM: HeatAlarmNotificationEvent,
    NotificationType.HOME_HEALTH: HomeHealthNotificationEvent,
    NotificationType.HOME_MONITORING: HomeMonitoringNotificationEvent,
    NotificationType.HOME_SECURITY: HomeSecurityNotificationEvent,
    NotificationType.IRRIGATION: IrrigationNotificationEvent,
    NotificationType.LIGHT_SENSOR: LightSensorNotificationEvent,
    NotificationType.PEST_CONTROL: PestControlNotificationEvent,
    NotificationType.POWER_MANAGEMENT: PowerManagementNotificationEvent,
    NotificationType.SIREN: SirenNotificationEvent,
    NotificationType.SMOKE_ALARM: SmokeAlarmNotificationEvent,
    NotificationType.SYSTEM: SystemNotificationEvent,
    NotificationType.WATER_ALARM: WaterAlarmNotificationEvent,
    NotificationType.WATER_QUALITY_MONITORING: WaterQualityMonitoringNotificationEvent,
    NotificationType.WATER_VALVE: WaterValveNotificationEvent,
    NotificationType.WEATHER_ALARM: WeatherAlarmNotificationEvent,
}

NOTIFICATION_EVENT_TO_EVENT_VALUE_MAP: dict[
    NotificationEvent, type[NotificationEventValue]
] = {
    AccessControlNotificationEvent.BARRIER_PERFORMING_INITIALIZATION_PROCESS: BarrierPerformingInitializationProcessNotificationEventValue,
    AccessControlNotificationEvent.BARRIER_SAFETY_BEAM_OBSTACLE: BarrierSafetyBeamObstacleNotificationEventValue,
    AccessControlNotificationEvent.BARRIER_VACATION_MODE: BarrierVacationModeNotificationEventValue,
    AccessControlNotificationEvent.DOOR_STATE_WINDOW_DOOR_IS_OPEN: DoorStateWindowDoorIsOpenNotificationEventValue,
    Co2AlarmNotificationEvent.TEST_STATUS_CARBON_DIOXIDE_TEST: TestStatusCarbonDioxideTestNotificationEventValue,
    CoAlarmNotificationEvent.TEST_STATUS_CARBON_MONOXIDE_TEST: TestStatusCarbonMonoxideTestNotificationEventValue,
    HomeHealthNotificationEvent.SLEEP_APNEA_STATUS_SLEEP_APNEA_DETECTED: SleepApneaStatusSleepApneaDetectedNotificationEventValue,
    HomeHealthNotificationEvent.VOC_LEVEL_STATUS_VOLATILE_ORGANIC_COMPOUND_LEVEL: VocLevelStatusVolatileOrganicCompoundLevelNotificationEventValue,
    WaterAlarmNotificationEvent.WATER_FLOW_ALARM: WaterFlowAlarmNotificationEventValue,
    WaterAlarmNotificationEvent.WATER_LEVEL_ALARM: WaterLevelAlarmNotificationEventValue,
    WaterAlarmNotificationEvent.WATER_PRESSURE_ALARM: WaterPressureAlarmNotificationEventValue,
    WaterAlarmNotificationEvent.WATER_TEMPERATURE_ALARM: WaterTemperatureAlarmNotificationEventValue,
    WaterQualityMonitoringNotificationEvent.ACIDITY_PH_STATUS_ACIDITY_PH_ALARM: AcidityStatusAcidityAlarmNotificationEventValue,
    WaterQualityMonitoringNotificationEvent.CHLORINE_ALARM: ChlorineAlarmNotificationEventValue,
    WaterQualityMonitoringNotificationEvent.WATER_OXIDATION_ALARM: WaterOxidationAlarmNotificationEventValue,
    WaterValveNotificationEvent.MASTER_VALVE_CURRENT_ALARM: MasterValveCurrentAlarmNotificationEventValue,
    WaterValveNotificationEvent.MASTER_VALVE_OPERATION: MasterValveOperationNotificationEventValue,
    WaterValveNotificationEvent.VALVE_CURRENT_ALARM: ValveCurrentAlarmNotificationEventValue,
    WaterValveNotificationEvent.VALVE_OPERATION: ValveOperationNotificationEventValue,
}


# ----------------------------------------------------------------------------------- #
# **END OF AUTOGENERATED CONTENT** (DO NOT EDIT/REMOVE THIS COMMENT BLOCK AND DO NOT  #
# EDIT ANYTHING ABOVE IT. IF A NEW IMPORT IS NEEDED, ADD IT TO THE IMPORTS IN THE     #
# CORRESPONDING GENERATION SCRIPT THEN RE-RUN THE SCRIPT. LINES WRITTEN BELOW THIS    #
# BLOCK WILL BE PRESERVED AS LONG AS THIS BLOCK REMAINS)                              #
# ----------------------------------------------------------------------------------- #



#!/usr/bin/env python3
"""Script to generate Multilevel Sensor CC constants."""

from __future__ import annotations

from collections import defaultdict
from collections.abc import Callable, Mapping
import pathlib

from const import AUTO_GEN_POST, AUTO_GEN_PRE
from helpers import (
    enum_name_format,
    format_for_class_name,
    get_json,
    get_manually_written_code,
    get_registry_location,
    run_black,
    separate_camel_case,
)

CONST_FILE_PATH = (
    pathlib.Path(__file__).parent.parent
    / "zwave_js_server/const/command_class/multilevel_sensor.py"
)


def normalize_scale_definition(scale_definitions: dict[str, dict]) -> dict[str, int]:
    """Convert a scales definition dictionary into a normalized dictionary."""
    scale_def_ = {}
    for scale_id, scale_props in scale_definitions.items():
        scale_name_ = enum_name_format(scale_props["label"], True)
        scale_def_[scale_name_] = scale_id

    return dict(sorted(scale_def_.items(), key=lambda kv: kv[0]))


scales = {}
sensors = {}

for sensor_props in get_json("sensors.json"):
    sensor_id = sensor_props["key"]
    scale_def = sensor_props["scales"]
    remove_parenthesis_ = True
    if sensor_id in (87, 88):
        remove_parenthesis_ = False
    sensor_name = enum_name_format(sensor_props["label"], remove_parenthesis_)
    sensors[sensor_name] = {"id": sensor_id, "label": sensor_props["label"]}
    scale_name = separate_camel_case(sensor_props.get("scaleGroupName", ""))
    if not (scale_name := enum_name_format(scale_name, remove_parenthesis_)):
        scale_name = sensor_name
    scales.setdefault(scale_name, {}).update(normalize_scale_definition(scale_def))
    sensors[sensor_name]["scale"] = scale_name

scales = dict(sorted(scales.items(), key=lambda kv: kv[0]))
sensors = dict(sorted(sensors.items(), key=lambda kv: kv[0]))


def generate_int_enum_class_definition(
    class_name: str,
    enum_map: Mapping[str, int | str | dict],
    enum_ref_url: str | None = None,
    get_id_func: Callable | None = None,
    docstring_info: str = "",
    base_class: str = "IntEnum",
) -> list[str]:
    """Generate an IntEnum class definition as an array of lines of string."""
    class_def: list[str] = []
    class_def.append(f"class {class_name}({base_class}):")
    docstring = (
        f'"""Enum for known {docstring_info} multilevel sensor types."""'.replace(
            "  ", " "
        )
    )
    class_def.append(f"    {docstring}")
    if enum_ref_url:
        class_def.append(f"    # {enum_ref_url}")
    for enum_name, enum_id in enum_map.items():
        if get_id_func:
            enum_id = get_id_func(enum_id)
        class_def.append(f"    {enum_name} = {enum_id}")
    return class_def


def generate_int_enum_base_class(class_name: str, docstring: str) -> list[str]:
    """Generate an IntEnum base class definition."""
    class_def: list[str] = []
    class_def.append(f"class {class_name}(IntEnum):")
    class_def.append(f"\t{docstring}")
    return class_def


SENSOR_TYPE_URL = get_registry_location("SensorTypes.ts")

lines = [
    '"""Constants for the Multilevel Sensor CC."""',
    *AUTO_GEN_PRE,
    "from __future__ import annotations",
    "",
    "from enum import IntEnum",
    'CC_SPECIFIC_SCALE = "scale"',
    'CC_SPECIFIC_SENSOR_TYPE = "sensorType"',
    "",
]

lines.extend(
    [f"{sensor}_PROPERTY = \"{data['label']}\"" for sensor, data in sensors.items()]
)

lines.extend(
    generate_int_enum_class_definition(
        "MultilevelSensorType",
        sensors,
        SENSOR_TYPE_URL,
        get_id_func=lambda x: x["id"],
    )
)

lines.extend(
    generate_int_enum_base_class(
        "MultilevelSensorScaleType",
        docstring='"""Common base class for multilevel sensor scale enums."""',
    )
)

_unit_name_to_enum_map = defaultdict(list)
for scale_name, scale_dict in scales.items():
    lines.extend(
        generate_int_enum_class_definition(
            format_for_class_name(scale_name, "Scale"),
            scale_dict,
            SENSOR_TYPE_URL,
            docstring_info=f"scales for {scale_name}",
            base_class="MultilevelSensorScaleType",
        )
    )
    for unit_name in scale_dict.keys():
        _unit_name_to_enum_map[unit_name].append(
            f"{format_for_class_name(scale_name, 'Scale')}.{unit_name}"
        )
unit_name_to_enum_map = dict(
    sorted(_unit_name_to_enum_map.items(), key=lambda kv: kv[0])
)
for unit_name, enum_list in unit_name_to_enum_map.items():
    unit_name_to_enum_map[unit_name] = sorted(enum_list)


multilevel_sensor_type_to_scale_map_line = (
    "MULTILEVEL_SENSOR_TYPE_TO_SCALE_MAP: dict[MultilevelSensorType, "
    "type[MultilevelSensorScaleType]] = {"
)
for sensor_name, sensor_def in sensors.items():
    multilevel_sensor_type_to_scale_map_line += (
        f"    MultilevelSensorType.{sensor_name}: "
        f"{format_for_class_name(sensor_def['scale'], 'Scale')},"
    )
multilevel_sensor_type_to_scale_map_line += "}"
lines.append(multilevel_sensor_type_to_scale_map_line)
lines.append("")

for unit_name, unit_enums in unit_name_to_enum_map.items():
    lines.append(
        f"UNIT_{unit_name}: list[MultilevelSensorScaleType] = [{','.join(sorted(unit_enums))}]"
    )

lines.extend(AUTO_GEN_POST)
lines.extend(get_manually_written_code(CONST_FILE_PATH))
CONST_FILE_PATH.write_text("\n".join(lines), encoding="utf-8")

run_black(CONST_FILE_PATH)



"""Constants for the Barrier Operator CC."""

from __future__ import annotations

from enum import IntEnum

SIGNALING_STATE_PROPERTY = "signalingState"


class BarrierEventSignalingSubsystemState(IntEnum):
    """Enum with all (known/used) Z-Wave Barrier Event Signaling Subsystem States."""

    # https://github.com/zwave-js/node-zwave-js/blob/15e1b59f627bfad8bfae114bd774a14089c76683/packages/zwave-js/src/lib/commandclass/BarrierOperatorCC.ts#L46-L49
    OFF = 0
    ON = 255


class BarrierState(IntEnum):
    """Enum with all (known/used) Z-Wave Barrier States."""

    # https://github.com/zwave-js/node-zwave-js/blob/15e1b59f627bfad8bfae114bd774a14089c76683/packages/zwave-js/src/lib/commandclass/BarrierOperatorCC.ts#L107-L113
    CLOSED = 0
    CLOSING = 252
    STOPPED = 253
    OPENING = 254
    OPEN = 255



"""Provide a model for the association."""

from __future__ import annotations

from dataclasses import dataclass, field
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .controller import Controller
    from .node import Node


@dataclass
class AssociationGroup:
    """Represent a association group dict type."""

    max_nodes: int
    is_lifeline: bool
    multi_channel: bool
    label: str
    profile: int | None = None
    issued_commands: dict[int, list[int]] = field(default_factory=dict)


@dataclass
class AssociationAddress:
    """Represent a association dict type."""

    controller: Controller
    node_id: int
    endpoint: int | None = None

    @property
    def node(self) -> Node | None:
        """Return the node."""
        if self.node_id in self.controller.nodes:
            return self.controller.nodes[self.node_id]
        return None



"""Test event helpers."""

from zwave_js_server import event


def test_once():
    """Test once listens to event once."""
    mock = event.EventBase()
    calls = []
    mock.once("test-event", calls.append)
    mock.emit("test-event", 1)
    mock.emit("test-event", 2)
    assert len(calls) == 1


def test_exception_on_emit(caplog):
    """Test exception on emit gets handled."""
    mock = event.EventBase()
    mock.on("test-event", lambda _: 1 / 0)
    mock.emit("test-event", 1)
    assert "Error handling event: test-event" in caplog.text



"""Provide a model for the Z-Wave JS controller's statistics."""

from __future__ import annotations

from contextlib import suppress
from dataclasses import dataclass, field
from typing import TYPE_CHECKING, TypedDict

from ..statistics import RouteStatistics, RouteStatisticsDataType

if TYPE_CHECKING:
    from ...client import Client


class ControllerLifelineRoutesDataType(TypedDict):
    """Represent a controller lifeline routes data dict type."""

    lwr: RouteStatisticsDataType
    nlwr: RouteStatisticsDataType


@dataclass
class ControllerLifelineRoutes:
    """Represent controller lifeline routes."""

    client: Client = field(repr=False)
    data: ControllerLifelineRoutesDataType = field(repr=False)
    lwr: RouteStatistics | None = field(init=False, default=None)
    nlwr: RouteStatistics | None = field(init=False, default=None)

    def __post_init__(self) -> None:
        """Post initialize."""
        if lwr := self.data.get("lwr"):
            with suppress(ValueError):
                self.lwr = RouteStatistics(self.client, lwr)
        if nlwr := self.data.get("nlwr"):
            with suppress(ValueError):
                self.nlwr = RouteStatistics(self.client, nlwr)


class ChannelRSSIDataType(TypedDict):
    """Represent a channel RSSI data dict type."""

    average: int
    current: int


class BackgroundRSSIDataType(TypedDict, total=False):
    """Represent a background RSSI data dict type."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/zwave-js/src/lib/controller/ControllerStatistics.ts#L40
    timestamp: int  # required
    channel0: ChannelRSSIDataType  # required
    channel1: ChannelRSSIDataType  # required
    channel2: ChannelRSSIDataType
    channel3: ChannelRSSIDataType


class ControllerStatisticsDataType(TypedDict, total=False):
    """Represent a controller statistics data dict type."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/zwave-js/src/lib/controller/ControllerStatistics.ts#L20-L39
    messagesTX: int  # required
    messagesRX: int  # required
    messagesDroppedTX: int  # required
    messagesDroppedRX: int  # required
    NAK: int  # required
    CAN: int  # required
    timeoutACK: int  # required
    timeoutResponse: int  # required
    timeoutCallback: int  # required
    backgroundRSSI: BackgroundRSSIDataType


@dataclass
class ChannelRSSI:
    """Represent a channel RSSI."""

    data: ChannelRSSIDataType = field(repr=False)
    average: int = field(init=False)
    current: int = field(init=False)

    def __post_init__(self) -> None:
        """Post initialize."""
        self.average = self.data["average"]
        self.current = self.data["current"]


@dataclass
class BackgroundRSSI:
    """Represent a background RSSI update."""

    data: BackgroundRSSIDataType = field(repr=False)
    timestamp: int = field(init=False)
    channel_0: ChannelRSSI = field(init=False)
    channel_1: ChannelRSSI = field(init=False)
    channel_2: ChannelRSSI | None = field(init=False)
    channel_3: ChannelRSSI | None = field(init=False)

    def __post_init__(self) -> None:
        """Post initialize."""
        self.timestamp = self.data["timestamp"]
        self.channel_0 = ChannelRSSI(self.data["channel0"])
        self.channel_1 = ChannelRSSI(self.data["channel1"])
        # Channels 2 and 3 may not be present, but 3 requires 2 to be present
        self.channel_2 = None
        self.channel_3 = None
        if channel_2 := self.data.get("channel2"):
            self.channel_2 = ChannelRSSI(channel_2)
            if channel_3 := self.data.get("channel3"):
                self.channel_3 = ChannelRSSI(channel_3)


@dataclass
class ControllerStatistics:
    """Represent a controller statistics update."""

    data: ControllerStatisticsDataType = field(repr=False)
    messages_tx: int = field(init=False)
    messages_rx: int = field(init=False)
    messages_dropped_rx: int = field(init=False)
    messages_dropped_tx: int = field(init=False)
    nak: int = field(init=False)
    can: int = field(init=False)
    timeout_ack: int = field(init=False)
    timeout_response: int = field(init=False)
    timeout_callback: int = field(init=False)
    background_rssi: BackgroundRSSI | None = field(init=False, default=None)

    def __post_init__(self) -> None:
        """Post initialize."""
        self.messages_tx = self.data["messagesTX"]
        self.messages_rx = self.data["messagesRX"]
        self.messages_dropped_rx = self.data["messagesDroppedRX"]
        self.messages_dropped_tx = self.data["messagesDroppedTX"]
        self.nak = self.data["NAK"]
        self.can = self.data["CAN"]
        self.timeout_ack = self.data["timeoutACK"]
        self.timeout_response = self.data["timeoutResponse"]
        self.timeout_callback = self.data["timeoutCallback"]
        if background_rssi := self.data.get("backgroundRSSI"):
            self.background_rssi = BackgroundRSSI(background_rssi)



"""Provide a model for the Z-Wave JS node's health checks and power tests."""

from __future__ import annotations

from dataclasses import dataclass, field
from typing import TypedDict

from ...const import PowerLevel


class LifelineHealthCheckResultDataType(TypedDict, total=False):
    """Represent a lifeline health check result data dict type."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/zwave-js/src/lib/node/Types.ts#L171
    latency: int  # required
    numNeighbors: int  # required
    failedPingsNode: int  # required
    rating: int  # required
    routeChanges: int
    minPowerlevel: int
    failedPingsController: int
    snrMargin: int


class LifelineHealthCheckSummaryDataType(TypedDict):
    """Represent a lifeline health check summary data dict type."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/zwave-js/src/lib/node/_Types.ts#L254
    results: list[LifelineHealthCheckResultDataType]
    rating: int


@dataclass
class LifelineHealthCheckResult:
    """Represent a lifeline health check result."""

    data: LifelineHealthCheckResultDataType = field(repr=False)
    latency: int = field(init=False)
    num_neighbors: int = field(init=False)
    failed_pings_node: int = field(init=False)
    rating: int = field(init=False)
    route_changes: int | None = field(init=False)
    min_power_level: PowerLevel | None = field(init=False, default=None)
    failed_pings_controller: int | None = field(init=False)
    snr_margin: int | None = field(init=False)

    def __post_init__(self) -> None:
        """Post initialize."""
        self.latency = self.data["latency"]
        self.num_neighbors = self.data["numNeighbors"]
        self.failed_pings_node = self.data["failedPingsNode"]
        self.rating = self.data["rating"]
        self.route_changes = self.data.get("routeChanges")
        if (min_power_level := self.data.get("minPowerlevel")) is not None:
            self.min_power_level = PowerLevel(min_power_level)
        self.failed_pings_controller = self.data.get("failedPingsController")
        self.snr_margin = self.data.get("snrMargin")


@dataclass
class LifelineHealthCheckSummary:
    """Represent a lifeline health check summary update."""

    data: LifelineHealthCheckSummaryDataType = field(repr=False)
    rating: int = field(init=False)
    results: list[LifelineHealthCheckResult] = field(init=False)

    def __post_init__(self) -> None:
        """Post initialize."""
        self.rating = self.data["rating"]
        self.results = [
            LifelineHealthCheckResult(r) for r in self.data.get("results", [])
        ]


class RouteHealthCheckResultDataType(TypedDict, total=False):
    """Represent a route health check result data dict type."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/zwave-js/src/lib/node/_Types.ts#L285
    numNeighbors: int  # required
    rating: int  # required
    failedPingsToTarget: int
    failedPingsToSource: int
    minPowerlevelSource: int
    minPowerlevelTarget: int


class RouteHealthCheckSummaryDataType(TypedDict):
    """Represent a route health check summary data dict type."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/zwave-js/src/lib/node/_Types.ts#L317
    results: list[RouteHealthCheckResultDataType]
    rating: int


@dataclass
class RouteHealthCheckResult:
    """Represent a route health check result."""

    data: RouteHealthCheckResultDataType = field(repr=False)
    num_neighbors: int = field(init=False)
    rating: int = field(init=False)
    failed_pings_to_target: int | None = field(init=False)
    failed_pings_to_source: int | None = field(init=False)
    min_power_level_source: PowerLevel | None = field(init=False, default=None)
    min_power_level_target: PowerLevel | None = field(init=False, default=None)

    def __post_init__(self) -> None:
        """Post initialize."""
        self.num_neighbors = self.data["numNeighbors"]
        self.rating = self.data["rating"]
        self.failed_pings_to_target = self.data.get("failedPingsToTarget")
        self.failed_pings_to_source = self.data.get("failedPingsToSource")
        if (min_power_level_source := self.data.get("minPowerlevelSource")) is not None:
            self.min_power_level_source = PowerLevel(min_power_level_source)
        if (min_power_level_target := self.data.get("minPowerlevelTarget")) is not None:
            self.min_power_level_target = PowerLevel(min_power_level_target)


@dataclass
class RouteHealthCheckSummary:
    """Represent a route health check summary update."""

    data: RouteHealthCheckSummaryDataType = field(repr=False)
    rating: int = field(init=False)
    results: list[RouteHealthCheckResult] = field(init=False)

    def __post_init__(self) -> None:
        """Post initialize."""
        self.rating = self.data["rating"]
        self.results = [RouteHealthCheckResult(r) for r in self.data.get("results", [])]


@dataclass
class TestPowerLevelProgress:
    """Class to represent a test power level progress update."""

    acknowledged: int
    total: int


@dataclass
class CheckHealthProgress:
    """Represent a check lifeline/route health progress update."""

    rounds: int
    total_rounds: int
    last_rating: int
    last_result: LifelineHealthCheckResult | RouteHealthCheckResult



"""Provide a model for inclusion and provisioning."""

from __future__ import annotations

from dataclasses import dataclass
from typing import Any, TypedDict

from ...const import Protocols, ProvisioningEntryStatus, QRCodeVersion, SecurityClass


class InclusionGrantDataType(TypedDict):
    """Representation of an inclusion grant data dict type."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/zwave-js/src/lib/controller/Inclusion.ts#L48-L56
    securityClasses: list[int]
    clientSideAuth: bool


@dataclass
class InclusionGrant:
    """Representation of an inclusion grant."""

    security_classes: list[SecurityClass]
    client_side_auth: bool

    def to_dict(self) -> InclusionGrantDataType:
        """Return InclusionGrantDataType dict from self."""
        return {
            "securityClasses": [sec_cls.value for sec_cls in self.security_classes],
            "clientSideAuth": self.client_side_auth,
        }

    @classmethod
    def from_dict(cls, data: InclusionGrantDataType) -> InclusionGrant:
        """Return InclusionGrant from InclusionGrantDataType dict."""
        return cls(
            security_classes=[
                SecurityClass(sec_cls) for sec_cls in data["securityClasses"]
            ],
            client_side_auth=data["clientSideAuth"],
        )


@dataclass
class ProvisioningEntry:
    """Class to represent the base fields of a provisioning entry."""

    dsk: str
    security_classes: list[SecurityClass]
    requested_security_classes: list[SecurityClass] | None = None
    status: ProvisioningEntryStatus = ProvisioningEntryStatus.ACTIVE
    protocol: Protocols | None = None
    additional_properties: dict[str, Any] | None = None

    def to_dict(self) -> dict[str, Any]:
        """Return PlannedProvisioning data dict from self."""
        data = {
            "dsk": self.dsk,
            "securityClasses": [sec_cls.value for sec_cls in self.security_classes],
            "status": self.status.value,
            **(self.additional_properties or {}),
        }
        if self.requested_security_classes:
            data["requestedSecurityClasses"] = [
                sec_cls.value for sec_cls in self.requested_security_classes
            ]
        if self.protocol is not None:
            data["protocol"] = self.protocol.value
        return data

    @classmethod
    def from_dict(cls, data: dict[str, Any]) -> ProvisioningEntry:
        """Return ProvisioningEntry from data dict."""
        cls_instance = cls(
            dsk=data["dsk"],
            security_classes=[
                SecurityClass(sec_cls) for sec_cls in data["securityClasses"]
            ],
        )
        if additional_properties := {
            k: v
            for k, v in data.items()
            if k not in ("dsk", "securityClasses", "requestedSecurityClasses", "status")
        }:
            cls_instance.additional_properties = additional_properties
        if "requestedSecurityClasses" in data:
            cls_instance.requested_security_classes = [
                SecurityClass(sec_cls) for sec_cls in data["requestedSecurityClasses"]
            ]
        if "status" in data:
            cls_instance.status = ProvisioningEntryStatus(data["status"])
        if "protocol" in data:
            cls_instance.protocol = Protocols(data["protocol"])
        return cls_instance


@dataclass
class QRProvisioningInformationMixin:
    """Mixin class to represent the base fields of a QR provisioning information."""

    version: QRCodeVersion
    generic_device_class: int
    specific_device_class: int
    installer_icon_type: int
    manufacturer_id: int
    product_type: int
    product_id: int
    application_version: str
    max_inclusion_request_interval: int | None
    uuid: str | None
    supported_protocols: list[Protocols] | None


@dataclass
class QRProvisioningInformation(ProvisioningEntry, QRProvisioningInformationMixin):
    """Representation of provisioning information retrieved from a QR code."""

    def to_dict(self) -> dict[str, Any]:
        """Return QRProvisioningInformation data dict from self."""
        data = {
            "version": self.version.value,
            "securityClasses": [sec_cls.value for sec_cls in self.security_classes],
            "dsk": self.dsk,
            "status": self.status.value,
            "genericDeviceClass": self.generic_device_class,
            "specificDeviceClass": self.specific_device_class,
            "installerIconType": self.installer_icon_type,
            "manufacturerId": self.manufacturer_id,
            "productType": self.product_type,
            "productId": self.product_id,
            "applicationVersion": self.application_version,
            **(self.additional_properties or {}),
        }
        if self.requested_security_classes:
            data["requestedSecurityClasses"] = [
                sec_cls.value for sec_cls in self.requested_security_classes
            ]
        if self.max_inclusion_request_interval is not None:
            data["maxInclusionRequestInterval"] = self.max_inclusion_request_interval
        if self.uuid is not None:
            data["uuid"] = self.uuid
        if self.supported_protocols is not None:
            data["supportedProtocols"] = [
                protocol.value for protocol in self.supported_protocols
            ]
        return data

    @classmethod
    def from_dict(cls, data: dict[str, Any]) -> QRProvisioningInformation:
        """Return QRProvisioningInformation from data dict."""
        supported_protocols: list[Protocols] | None = None
        if "supportedProtocols" in data:
            supported_protocols = [
                Protocols(supported_protocol)
                for supported_protocol in data["supportedProtocols"]
            ]
        cls_instance = cls(
            version=QRCodeVersion(data["version"]),
            security_classes=[
                SecurityClass(sec_cls) for sec_cls in data["securityClasses"]
            ],
            dsk=data["dsk"],
            generic_device_class=data["genericDeviceClass"],
            specific_device_class=data["specificDeviceClass"],
            installer_icon_type=data["installerIconType"],
            manufacturer_id=data["manufacturerId"],
            product_type=data["productType"],
            product_id=data["productId"],
            application_version=data["applicationVersion"],
            max_inclusion_request_interval=data.get("maxInclusionRequestInterval"),
            uuid=data.get("uuid"),
            supported_protocols=supported_protocols,
        )
        if additional_properties := {
            k: v
            for k, v in data.items()
            if k
            not in (
                "version",
                "securityClasses",
                "requestedSecurityClasses",
                "dsk",
                "genericDeviceClass",
                "specificDeviceClass",
                "installerIconType",
                "manufacturerId",
                "productType",
                "productId",
                "applicationVersion",
                "maxInclusionRequestInterval",
                "uuid",
                "supportedProtocols",
                "status",
            )
        }:
            cls_instance.additional_properties = additional_properties
        if "requestedSecurityClasses" in data:
            cls_instance.requested_security_classes = [
                SecurityClass(sec_cls) for sec_cls in data["requestedSecurityClasses"]
            ]
        if "status" in data:
            cls_instance.status = ProvisioningEntryStatus(data["status"])
        return cls_instance



"""Provide a model for the Z-Wave JS node's statistics."""

from __future__ import annotations

from contextlib import suppress
from dataclasses import dataclass, field
from datetime import datetime
from typing import TYPE_CHECKING, TypedDict

from zwave_js_server.exceptions import RssiErrorReceived

from ...const import RssiError
from ..statistics import RouteStatistics, RouteStatisticsDataType

if TYPE_CHECKING:
    from ...client import Client


class NodeStatisticsDataType(TypedDict, total=False):
    """Represent a node statistics data dict type."""

    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/zwave-js/src/lib/node/NodeStatistics.ts#L21-L33
    commandsTX: int  # required
    commandsRX: int  # required
    commandsDroppedTX: int  # required
    commandsDroppedRX: int  # required
    timeoutResponse: int  # required
    rtt: float
    rssi: int
    lwr: RouteStatisticsDataType
    nlwr: RouteStatisticsDataType
    lastSeen: str


@dataclass
class NodeStatistics:
    """Represent a node statistics update."""

    client: Client = field(repr=False)
    data: NodeStatisticsDataType = field(repr=False)
    commands_tx: int = field(init=False)
    commands_rx: int = field(init=False)
    commands_dropped_rx: int = field(init=False)
    commands_dropped_tx: int = field(init=False)
    timeout_response: int = field(init=False)
    rtt: float | None = field(init=False)
    lwr: RouteStatistics | None = field(init=False, default=None)
    nlwr: RouteStatistics | None = field(init=False, default=None)
    last_seen: datetime | None = field(init=False, default=None)

    def __post_init__(self) -> None:
        """Post initialize."""
        self.commands_tx = self.data["commandsTX"]
        self.commands_rx = self.data["commandsRX"]
        self.commands_dropped_rx = self.data["commandsDroppedRX"]
        self.commands_dropped_tx = self.data["commandsDroppedTX"]
        self.timeout_response = self.data["timeoutResponse"]
        self.rtt = self.data.get("rtt")
        if last_seen := self.data.get("lastSeen"):
            self.last_seen = datetime.fromisoformat(last_seen)
        if lwr := self.data.get("lwr"):
            with suppress(ValueError):
                self.lwr = RouteStatistics(self.client, lwr)
        if nlwr := self.data.get("nlwr"):
            with suppress(ValueError):
                self.nlwr = RouteStatistics(self.client, nlwr)

    @property
    def rssi(self) -> int | None:
        """
        Return average RSSI of frames received by this node.

        Consecutive non-error measurements are combined using an exponential moving
        average.
        """
        if not self.data or (rssi_ := self.data.get("rssi")) is None:
            return None
        if rssi_ in [item.value for item in RssiError]:
            raise RssiErrorReceived(RssiError(rssi_))
        return rssi_



#!/usr/bin/env python3
"""Script to generate Notification CC constants."""

from __future__ import annotations

from collections.abc import Callable, Mapping
import pathlib

from const import AUTO_GEN_POST, AUTO_GEN_PRE
from helpers import (
    enum_name_format,
    format_for_class_name,
    get_json,
    get_manually_written_code,
    get_registry_location,
    run_black,
)

CONST_FILE_PATH = (
    pathlib.Path(__file__).parent.parent
    / "zwave_js_server/const/command_class/notification.py"
)

notifications = {}
params = {}
for notification_payload in get_json("notifications.json"):
    notification_type = notification_payload["type"]
    notification_name = notification_payload["name"].title()
    notifications[notification_name] = {
        "type": notification_type,
        "events": {},
    }
    for event in notification_payload.get("variables", []):
        event_name = event["name"].title()
        states = notifications[notification_name]["events"]
        for state_props in event["states"].values():
            state_id = state_props["value"]
            state_name = state_props["label"].title()
            if (
                state_name.lower() not in event_name.lower()
                and event_name.lower() not in state_name.lower()
            ):
                state_name = f"{event_name} {state_name}"
            states[state_name] = state_id
            if (
                "parameter" in state_props
                and state_props["parameter"]["type"] == "enum"
            ):
                for enum_id, enum_name in state_props["parameter"]["values"].items():
                    enum_id = int(enum_id)
                    params.setdefault(notification_name, {}).setdefault(state_name, {})[
                        enum_name.title()
                    ] = enum_id
    for event_data in notification_payload.get("events", {}).values():
        event_id = event_data["value"]
        notifications[notification_name]["events"][
            event_data["label"].title()
        ] = event_id


notifications = dict(sorted(notifications.items(), key=lambda kv: kv[0]))
params = dict(sorted(params.items(), key=lambda kv: kv[0]))
for notification_name, enum_data in params.items():
    params[notification_name] = dict(sorted(enum_data.items(), key=lambda kv: kv[0]))


def generate_int_enum_class_definition(
    class_name: str,
    enum_map: Mapping[str, int | str | dict],
    enum_ref_url: str | None = None,
    get_id_func: Callable | None = None,
    docstring_info: str = "",
    base_class: str = "IntEnum",
) -> list[str]:
    """Generate an IntEnum class definition as an array of lines of string."""
    class_def: list[str] = []
    class_def.append(f"class {class_name}({base_class}):")
    docstring = f'"""Enum for known {docstring_info}."""'.replace("  ", " ")
    class_def.append(f"    {docstring}")
    if enum_ref_url:
        class_def.append(f"    # {enum_ref_url}")
    class_def.append("    UNKNOWN = -1")
    for _enum_name, _enum_id in sorted(enum_map.items(), key=lambda x: x[0]):
        if get_id_func:
            _enum_id = get_id_func(_enum_id)
        class_def.append(f"    {enum_name_format(_enum_name, False)} = {_enum_id}")
    class_def.extend(
        [
            "    @classmethod",
            f"    def _missing_(cls: type, value: object) -> {class_name}:  # noqa: ARG003",
            '        """Set default enum member if an unknown value is provided."""',
            f"        return {class_name}.UNKNOWN",
        ]
    )
    return class_def


def generate_int_enum_base_class(class_name: str, docstring: str) -> list[str]:
    """Generate an IntEnum base class definition."""
    class_def: list[str] = []
    class_def.append(f"class {class_name}(IntEnum):")
    class_def.append(f"    {docstring}")
    return class_def


NOTIFICATIONS_URL = get_registry_location("Notifications.ts")

lines = [
    "# pylint: disable=line-too-long",
    '"""Constants for the Notification CC."""',
    *AUTO_GEN_PRE,
    "from __future__ import annotations",
    "",
    "from enum import IntEnum",
    'CC_SPECIFIC_NOTIFICATION_TYPE = "notificationType"',
]

lines.extend(
    generate_int_enum_class_definition(
        "NotificationType",
        notifications,
        NOTIFICATIONS_URL,
        get_id_func=lambda x: x["type"],
        docstring_info="notification types",
    )
)

lines.extend(
    generate_int_enum_base_class(
        "NotificationEvent",
        docstring='"""Common base class for Notification CC states enums."""',
    )
)

lines.extend(
    generate_int_enum_base_class(
        "NotificationEventValue",
        docstring='"""Common base class for Notification CC state value enums."""',
    )
)

# Add events that have enums

_notification_type_to_notification_event_map = {}
_notification_event_to_event_value_map = {}
for notification_type, event_map in notifications.items():
    notification_event_name = f"{notification_type} Notification Event"
    lines.extend(
        generate_int_enum_class_definition(
            format_for_class_name(notification_event_name),
            event_map["events"],
            NOTIFICATIONS_URL,
            docstring_info=notification_event_name.lower(),
            base_class="NotificationEvent",
        )
    )
    _notification_type_to_notification_event_map[notification_type] = (
        format_for_class_name(notification_event_name)
    )
    if notification_type in params:
        for event_name, event_values in params[notification_type].items():
            notification_event_value_name = f"{event_name} Notification Event Value"
            lines.extend(
                generate_int_enum_class_definition(
                    format_for_class_name(notification_event_value_name),
                    event_values,
                    NOTIFICATIONS_URL,
                    docstring_info=notification_event_value_name.lower(),
                    base_class="NotificationEventValue",
                )
            )
            _notification_event_to_event_value_map[
                f"{format_for_class_name(notification_event_name)}.{enum_name_format(event_name, False)}"
            ] = format_for_class_name(notification_event_value_name)

notification_type_to_notification_event_map = dict(
    sorted(_notification_type_to_notification_event_map.items(), key=lambda kv: kv[0])
)
notification_type_to_event_map_line = (
    "NOTIFICATION_TYPE_TO_EVENT_MAP: dict[NotificationType, "
    "type[NotificationEvent]] = {"
)
for (
    notification_type,
    notification_event,
) in notification_type_to_notification_event_map.items():
    notification_type_to_event_map_line += f"    NotificationType.{enum_name_format(notification_type, False)}: {notification_event},"
notification_type_to_event_map_line += "}"
lines.append(notification_type_to_event_map_line)
lines.append("")


notification_event_to_event_value_map = dict(
    sorted(_notification_event_to_event_value_map.items(), key=lambda kv: kv[0])
)
notification_event_to_event_value_map_line = (
    "NOTIFICATION_EVENT_TO_EVENT_VALUE_MAP: dict[NotificationEvent, "
    "type[NotificationEventValue]] = {"
)
for (
    notification_event,
    notification_event_value,
) in notification_event_to_event_value_map.items():
    notification_event_to_event_value_map_line += (
        f"    {notification_event}: {notification_event_value},"
    )
notification_event_to_event_value_map_line += "}"
lines.append(notification_event_to_event_value_map_line)
lines.append("")

lines.extend(AUTO_GEN_POST)
lines.extend(get_manually_written_code(CONST_FILE_PATH))
CONST_FILE_PATH.write_text("\n".join(lines), encoding="utf-8")

run_black(CONST_FILE_PATH)



"""Test node utility functions."""

import copy
from unittest.mock import patch

import pytest

from zwave_js_server.const import CommandClass, CommandStatus
from zwave_js_server.exceptions import (
    BulkSetConfigParameterFailed,
    InvalidNewValue,
    NotFoundError,
    SetValueFailed,
    ValueTypeError,
)
from zwave_js_server.model.node import Node
from zwave_js_server.model.value import ConfigurationValue
from zwave_js_server.util.node import (
    async_bulk_set_partial_config_parameters,
    async_set_config_parameter,
    dump_node_state,
)


@pytest.mark.parametrize("endpoint", [0, 1])
async def test_configuration_parameter_values(
    endpoint,
    client,
    climate_radio_thermostat_ct100_plus_state,
    inovelli_switch_state,
    uuid4,
    mock_command,
):
    """Test node methods to get and set configuration parameter values."""
    node_state = copy.deepcopy(climate_radio_thermostat_ct100_plus_state)
    node_2_state = copy.deepcopy(inovelli_switch_state)
    # Put all config parameters on endpoint we are testing
    for state in (node_state, node_2_state):
        for value in state["values"]:
            if (
                value["commandClass"] == CommandClass.CONFIGURATION
                and value["endpoint"] == 0
            ):
                value["endpoint"] = endpoint

    node: Node = Node(client, node_state)
    node_2: Node = Node(client, node_2_state)
    ack_commands = mock_command(
        {"command": "node.set_value", "nodeId": node.node_id},
        {"result": {"status": 255}},
    )

    assert node.node_id == 13
    config_values = node.get_configuration_values()
    assert len(config_values) == 12

    assert node_2.node_id == 31
    config_values_2 = node_2.get_configuration_values()
    assert len(config_values_2) == 15

    for value in config_values.values():
        assert isinstance(value, ConfigurationValue)

    # Test setting a configuration parameter that has no metadata
    with pytest.raises(NotImplementedError):
        await async_set_config_parameter(node, 1, 2, endpoint=endpoint)

    # Test setting a manual entry configuration parameter with a valid value
    ack_commands_2 = mock_command(
        {"command": "node.set_value", "nodeId": node_2.node_id},
        {"result": {"status": 255}},
    )

    zwave_value, cmd_status = await async_set_config_parameter(
        node_2, 190, 8, 255, endpoint=endpoint
    )
    assert isinstance(zwave_value, ConfigurationValue)
    assert cmd_status.status == CommandStatus.ACCEPTED

    value = node_2.values[f"31-112-{endpoint}-8-255"]
    assert len(ack_commands_2) == 1
    assert ack_commands_2[0] == {
        "command": "node.set_value",
        "nodeId": node_2.node_id,
        "valueId": {
            "commandClass": 112,
            "endpoint": endpoint,
            "property": 8,
            "propertyKey": 255,
        },
        "value": 190,
        "messageId": uuid4,
    }

    zwave_value, cmd_status = await async_set_config_parameter(
        node_2, "Blue", 8, 255, endpoint=endpoint
    )
    assert isinstance(zwave_value, ConfigurationValue)
    assert cmd_status.status == CommandStatus.ACCEPTED

    value = node_2.values[f"31-112-{endpoint}-8-255"]
    assert len(ack_commands_2) == 2
    assert ack_commands_2[1] == {
        "command": "node.set_value",
        "nodeId": node_2.node_id,
        "valueId": {
            "commandClass": 112,
            "endpoint": endpoint,
            "property": 8,
            "propertyKey": 255,
        },
        "value": 170,
        "messageId": uuid4,
    }

    # Test configuration parameter not found when using an invalid property name
    with pytest.raises(NotFoundError):
        await async_set_config_parameter(
            node, 5, "fake configuration parameter name", endpoint=endpoint
        )

    # Test using an invalid state label to set a value
    with pytest.raises(InvalidNewValue):
        await async_set_config_parameter(node, "fake state label", 1, endpoint=endpoint)

    # Test configuration parameter not found when property key is invalid
    with pytest.raises(NotFoundError):
        await async_set_config_parameter(node, 1, 1, property_key=1, endpoint=endpoint)

    # Test setting a configuration parameter by state label and property name
    zwave_value, cmd_status = await async_set_config_parameter(
        node, "2.0\u00b0 F", "Temperature Reporting Threshold", endpoint=endpoint
    )
    assert isinstance(zwave_value, ConfigurationValue)
    assert cmd_status.status == CommandStatus.ACCEPTED

    value = node.values[f"13-112-{endpoint}-1"]
    assert len(ack_commands) == 3
    assert ack_commands[2] == {
        "command": "node.set_value",
        "nodeId": node.node_id,
        "valueId": {
            "commandClass": 112,
            "endpoint": endpoint,
            "property": 1,
        },
        "value": 4,
        "messageId": uuid4,
    }


@pytest.mark.parametrize("endpoint", [0, 1])
async def test_bulk_set_partial_config_parameters(
    endpoint, client, multisensor_6_state, uuid4, mock_command
):
    """Test bulk setting partial config parameters."""
    node_state = copy.deepcopy(multisensor_6_state)
    # Put all config parameters on endpoint we are testing
    for value in node_state["values"]:
        if (
            value["commandClass"] == CommandClass.CONFIGURATION
            and value["endpoint"] == 0
        ):
            value["endpoint"] = endpoint
    node: Node = Node(client, node_state)
    ack_commands = mock_command(
        {"command": "node.set_value", "nodeId": node.node_id},
        {"result": {"status": 255}},
    )
    cmd_status = await async_bulk_set_partial_config_parameters(
        node, 101, 241, endpoint=endpoint
    )
    assert cmd_status.status == CommandStatus.QUEUED
    assert len(ack_commands) == 1
    assert ack_commands[0] == {
        "command": "node.set_value",
        "nodeId": node.node_id,
        "valueId": {
            "commandClass": CommandClass.CONFIGURATION.value,
            "endpoint": endpoint,
            "property": 101,
        },
        "value": 241,
        "messageId": uuid4,
    }

    cmd_status = await async_bulk_set_partial_config_parameters(
        node, 101, {128: 1, 64: 1, 32: 1, 16: 1, 1: 1}, endpoint=endpoint
    )
    assert cmd_status.status == CommandStatus.QUEUED
    assert len(ack_commands) == 2
    assert ack_commands[1] == {
        "command": "node.set_value",
        "nodeId": node.node_id,
        "valueId": {
            "commandClass": CommandClass.CONFIGURATION.value,
            "endpoint": endpoint,
            "property": 101,
        },
        "value": 241,
        "messageId": uuid4,
    }

    # Only set some values so we use cached values for the rest
    cmd_status = await async_bulk_set_partial_config_parameters(
        node, 101, {64: 1, 32: 1, 16: 1, 1: 1}, endpoint=endpoint
    )
    assert cmd_status.status == CommandStatus.QUEUED
    assert len(ack_commands) == 3
    assert ack_commands[2] == {
        "command": "node.set_value",
        "nodeId": node.node_id,
        "valueId": {
            "commandClass": CommandClass.CONFIGURATION.value,
            "endpoint": endpoint,
            "property": 101,
        },
        "value": 241,
        "messageId": uuid4,
    }

    # Use property key names instead of bitmasks for dict key
    cmd_status = await async_bulk_set_partial_config_parameters(
        node,
        101,
        {
            "Group 1: Send humidity reports": 1,
            "Group 1: Send temperature reports": 1,
            "Group 1: Send ultraviolet reports": 1,
            "Group 1: Send battery reports": 1,
        },
        endpoint=endpoint,
    )
    assert cmd_status.status == CommandStatus.QUEUED
    assert len(ack_commands) == 4
    assert ack_commands[3] == {
        "command": "node.set_value",
        "nodeId": node.node_id,
        "valueId": {
            "commandClass": CommandClass.CONFIGURATION.value,
            "endpoint": endpoint,
            "property": 101,
        },
        "value": 241,
        "messageId": uuid4,
    }

    # Use an invalid property
    with pytest.raises(NotFoundError):
        await async_bulk_set_partial_config_parameters(node, 999, 99, endpoint=endpoint)

    # use an invalid bitmask
    with pytest.raises(NotFoundError):
        await async_bulk_set_partial_config_parameters(
            node, 101, {128: 1, 64: 1, 32: 1, 16: 1, 2: 1}, endpoint=endpoint
        )

    # use an invalid property name
    with pytest.raises(NotFoundError):
        await async_bulk_set_partial_config_parameters(
            node, 101, {"Invalid property name": 1}, endpoint=endpoint
        )

    # use an invalid state value
    with pytest.raises(BulkSetConfigParameterFailed):
        await async_bulk_set_partial_config_parameters(
            node, 101, {128: "fake state", 64: 1, 32: 1, 16: 1, 1: 1}, endpoint=endpoint
        )

    # Try to bulkset a property that isn't broken into partials with a dictionary
    with pytest.raises(ValueTypeError):
        await async_bulk_set_partial_config_parameters(
            node, 252, {1: 1}, endpoint=endpoint
        )

    # Try to bulkset a property that isn't broken into partials, it should fall back to
    # async_set_config_parameter
    with patch("zwave_js_server.util.node.async_set_config_parameter") as mock_cmd:
        mock_cmd.return_value = (None, None)
        await async_bulk_set_partial_config_parameters(node, 252, 1, endpoint=endpoint)
        mock_cmd.assert_called_once()


@pytest.mark.parametrize("endpoint", [0, 1])
async def test_bulk_set_with_full_and_partial_parameters(
    endpoint, client, partial_and_full_parameter_state, uuid4, mock_command
):
    """Test bulk setting config parameters when state has full and partial values."""
    node_state = copy.deepcopy(partial_and_full_parameter_state)
    # Put all config parameters on endpoint we are testing
    for value in node_state["values"]:
        if (
            value["commandClass"] == CommandClass.CONFIGURATION
            and value["endpoint"] == 0
        ):
            value["endpoint"] = endpoint
    node: Node = Node(client, node_state)
    ack_commands = mock_command(
        {"command": "node.set_value", "nodeId": node.node_id},
        {"result": {"status": 255}},
    )

    cmd_status = await async_bulk_set_partial_config_parameters(
        node, 8, 34867929, endpoint=endpoint
    )

    assert cmd_status.status == CommandStatus.ACCEPTED
    assert len(ack_commands) == 1
    assert ack_commands[0] == {
        "command": "node.set_value",
        "nodeId": node.node_id,
        "valueId": {
            "commandClass": CommandClass.CONFIGURATION.value,
            "endpoint": endpoint,
            "property": 8,
        },
        "value": 34867929,
        "messageId": uuid4,
    }


@pytest.mark.parametrize("endpoint", [0, 1])
async def test_failures(endpoint, client, multisensor_6_state, mock_command):
    """Test setting config parameter failures."""
    node_state = copy.deepcopy(multisensor_6_state)
    # Put all config parameters on endpoint we are testing
    for value in node_state["values"]:
        if (
            value["commandClass"] == CommandClass.CONFIGURATION
            and value["endpoint"] == 0
        ):
            value["endpoint"] = endpoint
    node: Node = Node(client, node_state)
    # We need the node to be alive so we wait for a response
    node.handle_alive(node)

    mock_command(
        {"command": "node.set_value", "nodeId": node.node_id},
        {"result": {"status": 0}},
    )

    with pytest.raises(SetValueFailed):
        await async_bulk_set_partial_config_parameters(
            node, 101, {64: 1, 32: 1, 16: 1, 1: 1}, endpoint=endpoint
        )

    with pytest.raises(SetValueFailed):
        await async_set_config_parameter(node, 1, 101, 64, endpoint=endpoint)


@pytest.mark.parametrize("endpoint", [0, 1])
async def test_returned_values(endpoint, client, multisensor_6_state, mock_command):
    """Test returned values from setting config parameters."""
    node_state = copy.deepcopy(multisensor_6_state)
    # Put all config parameters on endpoint we are testing
    for value in node_state["values"]:
        if (
            value["commandClass"] == CommandClass.CONFIGURATION
            and value["endpoint"] == 0
        ):
            value["endpoint"] = endpoint
    node: Node = Node(client, node_state)
    # We need the node to be alive so we wait for a response
    node.handle_alive(node)

    mock_command(
        {"command": "node.set_value", "nodeId": node.node_id},
        {"result": {"status": 255}},
    )

    cmd_status = await async_bulk_set_partial_config_parameters(
        node, 101, {64: 1, 32: 1, 16: 1, 1: 1}, endpoint=endpoint
    )
    assert cmd_status.status == CommandStatus.ACCEPTED

    zwave_value, cmd_status = await async_set_config_parameter(
        node, 1, 101, 64, endpoint=endpoint
    )
    assert isinstance(zwave_value, ConfigurationValue)
    assert cmd_status.status == CommandStatus.ACCEPTED


async def test_dump_node_state(inovelli_switch, inovelli_switch_state):
    """Test dumping the node state."""
    assert inovelli_switch_state != inovelli_switch.data
    assert "values" not in inovelli_switch.data
    assert "endpoints" not in inovelli_switch.data
    dump_state = dump_node_state(inovelli_switch)
    for val in dump_state["values"].values():
        assert val in inovelli_switch_state["values"]
    for endpoint in dump_state["endpoints"].values():
        assert endpoint in inovelli_switch_state["endpoints"]



"""Multilevel Sensor Command Class specific utility functions for values."""

from __future__ import annotations

from ...const import CommandClass
from ...const.command_class.multilevel_sensor import (
    CC_SPECIFIC_SCALE,
    CC_SPECIFIC_SENSOR_TYPE,
    MULTILEVEL_SENSOR_TYPE_TO_SCALE_MAP,
    MultilevelSensorScaleType,
    MultilevelSensorType,
)
from ...exceptions import InvalidCommandClass, UnknownValueData
from ...model.value import Value


def get_multilevel_sensor_type(value: Value) -> MultilevelSensorType:
    """Get the MultilevelSensorType for a given value."""
    if value.command_class != CommandClass.SENSOR_MULTILEVEL:
        raise InvalidCommandClass(value, CommandClass.SENSOR_MULTILEVEL)
    try:
        return MultilevelSensorType(value.metadata.cc_specific[CC_SPECIFIC_SENSOR_TYPE])
    except ValueError:
        raise UnknownValueData(
            value, f"metadata.cc_specific.{CC_SPECIFIC_SENSOR_TYPE}"
        ) from None


def get_multilevel_sensor_scale_type(value: Value) -> MultilevelSensorScaleType:
    """Get the ScaleType for a given value."""
    sensor_type = get_multilevel_sensor_type(value)
    scale_enum = MULTILEVEL_SENSOR_TYPE_TO_SCALE_MAP[sensor_type]
    try:
        return scale_enum(value.metadata.cc_specific[CC_SPECIFIC_SCALE])
    except ValueError:
        raise UnknownValueData(
            value, f"metadata.cc_specific.{CC_SPECIFIC_SCALE}"
        ) from None



"""Test the CLI in main."""

import sys
from unittest.mock import AsyncMock, patch

import pytest

from zwave_js_server.__main__ import main

from .common import update_ws_client_msg_queue

# pylint: disable=unused-argument


@pytest.fixture(name="client_session")
def client_session_fixture(ws_client):
    """Mock the aiohttp client session."""
    with patch("aiohttp.ClientSession") as session:
        session.return_value.__aenter__.return_value.ws_connect.side_effect = AsyncMock(
            return_value=ws_client
        )
        yield session


def test_server_version(client_session, url, ws_client, result, capsys):
    """Test print server version."""
    with (
        patch.object(sys, "argv", ["zwave_js_server", url, "--server-version"]),
        pytest.raises(SystemExit) as sys_exit,
    ):
        main()

    assert sys_exit.value.code == 0
    captured = capsys.readouterr()
    assert captured.out == (
        "Driver: test_driver_version\n"
        "Server: test_server_version\n"
        "Home ID: test_home_id\n"
    )
    assert ws_client.receive_json.call_count == 1
    assert ws_client.close.call_count == 1


@pytest.mark.parametrize("result", ["test_result"])
def test_dump_state(
    client_session, url, ws_client, result, version_data, initialize_data, capsys
):
    """Test dump state."""
    update_ws_client_msg_queue(ws_client, (version_data, initialize_data, result))
    with (
        patch.object(sys, "argv", ["zwave_js_server", url, "--dump-state"]),
        pytest.raises(SystemExit) as sys_exit,
    ):
        main()

    assert sys_exit.value.code == 0
    captured = capsys.readouterr()
    assert captured.out == (
        "{'type': 'version', 'driverVersion': 'test_driver_version', "
        "'serverVersion': 'test_server_version', 'homeId': 'test_home_id', "
        "'minSchemaVersion': 0, 'maxSchemaVersion': 44}\n"
        "{'type': 'result', 'success': True, 'result': {}, 'messageId': 'initialize'}\n"
        "test_result\n"
    )

    assert ws_client.receive_json.call_count == 3
    assert ws_client.close.call_count == 1


def test_connect(client_session, url, ws_client):
    """Test connect."""
    with (
        patch.object(sys, "argv", ["zwave_js_server", url]),
        pytest.raises(SystemExit) as sys_exit,
    ):
        main()

    assert sys_exit.value.code == 0
    assert ws_client.receive.call_count == 5



"""Test general utility functions."""

import pytest

from zwave_js_server.const import (
    Protocols,
    ProvisioningEntryStatus,
    QRCodeVersion,
    SecurityClass,
)
from zwave_js_server.model.utils import (
    async_parse_qr_code_string,
    async_try_parse_dsk_from_qr_code_string,
)


async def test_parse_qr_code_string(client, mock_command, uuid4):
    """Test parsing a QR code string."""
    ack_commands = mock_command(
        {"command": "utils.parse_qr_code_string"},
        {
            "qrProvisioningInformation": {
                "version": 0,
                "securityClasses": [0, 1, 2],
                "requestedSecurityClasses": [0],
                "status": 1,
                "dsk": "test",
                "genericDeviceClass": 1,
                "specificDeviceClass": 1,
                "installerIconType": 1,
                "manufacturerId": 1,
                "productType": 1,
                "productId": 1,
                "applicationVersion": "test",
                "maxInclusionRequestInterval": 1,
                "uuid": "test",
                "supportedProtocols": [0],
            }
        },
    )
    qr_provisioning_info = await async_parse_qr_code_string(
        client, "90testtesttesttesttesttesttesttesttesttesttesttesttest"
    )
    assert ack_commands[0] == {
        "command": "utils.parse_qr_code_string",
        "qr": "90testtesttesttesttesttesttesttesttesttesttesttesttest",
        "messageId": uuid4,
    }
    assert qr_provisioning_info.version == QRCodeVersion.S2
    assert set(qr_provisioning_info.security_classes) == {
        SecurityClass.S2_ACCESS_CONTROL,
        SecurityClass.S2_AUTHENTICATED,
        SecurityClass.S2_UNAUTHENTICATED,
    }
    assert qr_provisioning_info.requested_security_classes == [
        SecurityClass.S2_UNAUTHENTICATED
    ]
    assert qr_provisioning_info.status == ProvisioningEntryStatus.INACTIVE
    assert (
        qr_provisioning_info.dsk
        == qr_provisioning_info.application_version
        == qr_provisioning_info.uuid
        == "test"
    )
    assert (
        qr_provisioning_info.generic_device_class
        == qr_provisioning_info.specific_device_class
        == qr_provisioning_info.installer_icon_type
        == qr_provisioning_info.manufacturer_id
        == qr_provisioning_info.product_type
        == qr_provisioning_info.product_id
        == qr_provisioning_info.max_inclusion_request_interval
        == 1
    )
    assert qr_provisioning_info.supported_protocols == [Protocols.ZWAVE]

    # Test invalid QR code length fails
    with pytest.raises(ValueError):
        await async_parse_qr_code_string(client, "test")


async def test_async_try_parse_dsk_from_qr_code_string(client, mock_command, uuid4):
    """Test trying to parse a DSK from a qr code string."""
    ack_commands = mock_command(
        {"command": "utils.try_parse_dsk_from_qr_code_string"}, {"dsk": "abc"}
    )
    dsk = await async_try_parse_dsk_from_qr_code_string(
        client, "90testtesttesttesttesttesttesttesttesttesttesttesttest"
    )
    assert ack_commands[0] == {
        "command": "utils.try_parse_dsk_from_qr_code_string",
        "qr": "90testtesttesttesttesttesttesttesttesttesttesttesttest",
        "messageId": uuid4,
    }
    assert dsk == "abc"


async def test_async_try_parse_dsk_from_qr_code_string_fails(
    client, mock_command, uuid4
):
    """Test trying to parse a DSK from a qr code string fails."""

    ack_commands = mock_command(
        {"command": "utils.try_parse_dsk_from_qr_code_string"}, {}
    )

    dsk = await async_try_parse_dsk_from_qr_code_string(
        client, "90testtesttesttesttesttesttesttesttesttesttesttesttest"
    )
    assert ack_commands[0] == {
        "command": "utils.try_parse_dsk_from_qr_code_string",
        "qr": "90testtesttesttesttesttesttesttesttesttesttesttesttest",
        "messageId": uuid4,
    }
    assert dsk is None



"""Version helper."""

from __future__ import annotations

import aiohttp

from .model.version import VersionInfo


async def get_server_version(url: str, session: aiohttp.ClientSession) -> VersionInfo:
    """Return a server version."""
    client = await session.ws_connect(url)
    try:
        return VersionInfo.from_message(await client.receive_json())
    finally:
        await client.close()
